import org.w3c.tools.resources.ResourceReference;
import java.util.*;
import java.util.Map.Entry;

aspect LORAspect {
    
    public class Pair
    {
        public Pair( Object o1, Object o2 ) { this.o1 = o1; this.o2 = o2; }
        public int hashCode() {
            return o1.hashCode() + o2.hashCode();
        }
        public boolean equals( Object other ) {
            if( other instanceof Pair ) {
                Pair p = (Pair) other;
                return o1.equals( p.o1 ) && o2.equals( p.o2 );
            } else return false;
        }
        public String toString() {
            return "Pair ("+o1+","+o2+")";
        }
        public Object getO1() { return o1; }
        public Object getO2() { return o2; }

        protected Object o1;
        protected Object o2;
    }

	pointcut lock(Thread t, org.w3c.tools.resources.ResourceReference r) :
		Logger.lock() && target(r) && let(t,Thread.currentThread());

	pointcut unlock(Thread t,org.w3c.tools.resources.ResourceReference r) :
		Logger.unlock() && target(r) && let(t,Thread.currentThread());

    WeakIdentityHashMap threadToRes = new WeakIdentityHashMap(); 
    WeakIdentityHashMap threadToPairs = new WeakIdentityHashMap();
    
	synchronized before(Thread t, org.w3c.tools.resources.ResourceReference r): lock(t,r) {
	    Set res = (Set) threadToRes.get(t);
        if(res==null) {
            res = new WeakIdentityHashSet();
            threadToRes.put(t,res);
        }
        
        for (Iterator iter = res.iterator(); iter.hasNext();) {
            ResourceReference re = (ResourceReference) iter.next();
            
            Set pairs = (Set) threadToPairs.get(t);
            if(pairs==null) {
                pairs = new HashSet();
                threadToPairs.put(t,pairs);
            }
            pairs.add(new Pair(re,r));
            
            Pair inverse = new Pair(r,re);
            for (Iterator threadIter = threadToPairs.entrySet().iterator(); threadIter.hasNext();) {
                Entry entry = (Entry) threadIter.next();
                if(((Set)entry.getValue()).contains(inverse) && entry.getKey()!=t) {
                    new RuntimeException("Lock order reversal detected (Plain AJ)").printStackTrace();
                }
            }
        }
        res.add(r);
    }

    synchronized before(Thread t, org.w3c.tools.resources.ResourceReference r): unlock(t,r) {
        Set res = (Set) threadToRes.get(t);
        if(res!=null)        
            res.remove(r);        
    }
}
