package itds;

import java.util.*;
import java.lang.ref.WeakReference;

// roll our own vectors, with a special field for
// keeping track of enumerations
class MyVector extends Vector {
   List enumerations = new LinkedList();
   public MyVector() { super(); }
   public MyVector(int n) { super(n); }
}

public aspect FailSafeEnum {

        pointcut vector_update() :
	call(* Vector.add*(..)) ||
	call(* Vector.clear()) ||
        call(* Vector.insertElementAt(..)) ||
        call(* Vector.remove*(..)) ||
        call(* Vector.retainAll(..)) ||
        call(* Vector.set*(..));

	/* --------------------------------------------------
           Vector is not weavable, so we replace all vectors
           in JHotDraw by our own subclass. This requires that
           we know that no Vector is constructed outside the
           weavable classes... check this as follows: */

	declare warning : 
	   call(Vector (!CH..*).*(..)) :
	    "Vector constructed outside JHotDraw";



        // intercept all Vector constructor calls and use our own
        Vector around(int n) : call(Vector.new(int)) && args(n) 
                               && !within(FailSafeEnum) {
            return new MyVector(n);
        }

	// also without arguments 
	Vector around() : call(Vector.new()) && !within(FailSafeEnum) {
            return new MyVector();
        }    
        
	/* end of hack to make Vector weavable
           ----------------------------------------------------- */



        /* Enumeration is not weavable, so we introduce a dummy interface */
        /* crude check to see whether that is possible: */

	declare warning : call(((!CH..*) && Enumeration+).new(Vector)) :
	        "Constructor call on enumeration outside JHotDraw";
	
        public interface MyEnumeration extends Enumeration {}
        /* and put that on everything that implements Enumeration already */
	declare parents : Enumeration+ implements MyEnumeration;

        // tag all enumerations with state 
        private int MyEnumeration.state;
        private void MyEnumeration.setState(int n) { state = n; }
        private int MyEnumeration.getState() { return state; }

        // don't do too much cleaning up of weak-refs: we'll only
        // walk over the set once in every 10 creations

        private int count;

        // synced is important here as we modify the list of enumerations
        synchronized after(MyVector ds) returning(MyEnumeration e) :
	    call(Enumeration+.new(..)) && args(ds) {
            e.setState(0);
            ds.enumerations.add(new WeakReference(e));
            // clean up the list of enumerations only once every 10 times
            // not strictly necessary as the list will die with the MyVector
            // instance
	    if (count > 0)
		count--;
	    else { 
                count = 10; 
                for (Iterator it = ds.enumerations.iterator();
                   it.hasNext(); ) {
		WeakReference wr = (WeakReference)it.next();
                Object r = wr.get();
                if (r == null)
                    { it.remove(); }
	        }
	       	} 
        }

      

        synchronized after(MyVector ds) :
            vector_update() && target(ds) {
            for (Iterator it = ds.enumerations.iterator();
                 it.hasNext(); ) {
		WeakReference wr = (WeakReference)it.next();
                Object r = wr.get();
                if (r == null)
                    { it.remove(); }
                else {
                    MyEnumeration e = (MyEnumeration) r;
                    e.setState(1);
                }
	    }
        }

     
    
	synchronized before(MyEnumeration e) :          
	    call(Object Enumeration.nextElement()) &&
            target(e) {
	    if (e.getState() == 1)
		{//throw new ConcurrentModificationException();
		System.out.println("Violation detected!!");}
        }

       
                
     
}
   
