package benchmarks.hashset;

import java.util.*;

/**
 * Tracematch version of the hashcode tests.
 * 
 * @author Eric Bodden
 */
public aspect HashSetTest {

    pointcut addCollection(HashSet s, Collection c): call(* Collection.add(Object)) && target(s) && args(c);
    pointcut removeCollection(HashSet s, Collection c): call(* Collection.remove(Object)) && target(s) && args(c);
    pointcut modifyCollection(Collection c): (call(* Collection.add(..)) || call(* Collection.addAll(..)) || call(* Collection.remove(..)) || call(* Collection.removeAll(..))) && target(c);

    tracematch(HashSet s, Collection c) {
        
        sym add after: addCollection(s,c);
        sym remove after: removeCollection(s,c);
        sym modify before: modifyCollection(c);

        //G(add(s,c) -> !modify(c) U remove(s,c))
        add modify {
        
            System.out.println("Modified collection which is contained in HashSet."+
              "Next call to HashSet.contains(Object) may return false results.");

        }

    }
    
}

class Test {
    
    public static void main(String[] args) {
        Set s = new HashSet();        
        Collection c = new ArrayList();
        
        s.add(c);
        s.remove(c);
        System.out.println("allowed modification");
        c.add("allowed modification");
        
        s.add(c);
        System.out.println("this should trigger an error");
        c.add("this should trigger an error");
        
    }
    
}
