package benchmarks.hashset;

import java.util.*;

/**
 * This is a Set implementation with all functionality implemented by tracematches!
 * It is based on an empty "no-op" Marker class implementing <code>Set</code>.
 * All calles to <code>contains()</code> are rerouted by around-tracematches,
 * implementing the correct functionality.
 * 
 * Drawbacks: One cannot iterato over such a set.
 * 
 * @author Eric Bodden
 */
public aspect TMSet {

    /**
     * Test method.
     */
    public static void main(String[] args) {
        
        Set set = new NoOpSet();
        
        set.add("abc");
        System.out.println("This should return 'true': "+set.contains("abc"));
        set.remove("abc");
        System.out.println("This should return 'false': "+set.contains("abc"));        
    }
    

    /**
     * Handles all cases where something was added to the "Set" and not yet removed.
     * This results in <code>true</code>.
     */
    boolean tracematch(Set s, Object o) {
        
        sym add before: call(* Collection.add(Object)) && target(s) && args(o);
        sym remove before: call(* Collection.remove(Object)) && target(s) && args(o);
        sym contains around(s,o): call(* Collection.contains(..)) && target(s) && args(o);

        add contains {
            return true;
        }
    }
    
    /**
     * Handles all other cases. This results in <code>false</code>.
     */
    boolean tracematch() {
        
        sym contains around: call(* Collection.contains(..)) && target(HashSet);

        contains {
            return false;
        }
    }
    
    /**
     * A completely empty Set implementation.
     * 
     * @author Eric Bodden
     */    
    static class NoOpSet implements Set {

        public boolean add(Object o) {

            return false;
        }

        public boolean addAll(Collection c) {

            return false;
        }

        public void clear() {

        }

        public boolean contains(Object o) {

            return false;
        }

        public boolean containsAll(Collection c) {

            return false;
        }

        public boolean isEmpty() {

            return false;
        }

        public Iterator iterator() {

            return null;
        }

        public boolean remove(Object o) {

            return false;
        }

        public boolean removeAll(Collection c) {

            return false;
        }

        public boolean retainAll(Collection c) {

            return false;
        }

        public int size() {

            return 0;
        }

        public Object[] toArray() {

            return null;
        }

        public Object[] toArray(Object[] a) {

            return null;
        }
        
    }
}
