import java.lang.reflect.*;
import org.aspectj.lang.*;

/**
 * Checks for the origin of a NullPointer (for fields).
 * 
 * @author Eric Bodden
 */
public aspect NullPointerCheck {
    
    pointcut fieldSet(String fieldID, Object arg):
        set(* *.*)
     && args(arg)
     && let(fieldID, thisJoinPoint.getSignature().toString().intern());
    
    pointcut fieldSetToNull(String fieldID, JoinPoint jp, Object arg):
           fieldSet(fieldID,arg)
        && let(jp, thisJoinPoint)
        && if(arg==null);
    
    pointcut fieldAccess(String fieldID): get(* *.*) && let(fieldID, thisJoinPoint.getSignature().toString().intern());
    
    tracematch(String fieldID, JoinPoint jp) {
        
        sym fieldNull after: fieldSetToNull(fieldID,jp,*);
        sym fieldSet after: fieldSet(fieldID,*); //filter field sets -> beware, the symbols are overlapping!
        sym fieldAccess before: fieldAccess(fieldID);
    
        fieldNull /*no fieldSet in between */ fieldAccess {
            System.out.println("In the next step we will get a NullPointException caused by "+jp);
        }
        
        
    }
    
}

class NullPointerCheckTestEric {
    
    static String s = "";
    
    public static void main(String[] args) {
        NullPointerCheckTest foo = new NullPointerCheckTest();
        foo.s = null;
        System.out.println(foo.s);
    }
}
