import java.util.Map;
import java.util.Hashtable;

/**
 * Native AspectJ aspect for the "general" hashcode benchmark (the one which works on *any* hashmap)
 * 
 * @author Eric Bodden
 */
public aspect RealHashtableNativeAspect
{
    pointcut usemethod() :    call(* Map.get(..))
                     || call(* Map.remove(..))
                     || call(* Map.containsKey(..));

 
    pointcut put(Hashtable t, Object o):
        call(* Map.put(..))
        && target(t) && args(o,*);

    pointcut remove(Hashtable t, Object o):
        call(* Map.remove(Object))
        && target(t) && args(o);

    pointcut use(Hashtable t, Object o):
        usemethod() && target(t) && args(o);

    pointcut scope(): !within(RealHashSetNativeAspect); 

    
    WeakIdentityHashMap map /* hashmap -> (object -> hashcode) */
        = new WeakIdentityHashMap();
    
    after(Hashtable t, Object o) returning(): scope() && put(t, o) {
        Object val = map.get(t);
        if(val==null) {
            val = new WeakIdentityHashMap(); /* object -> hashcode */
            map.put(t, val);
        }
        WeakIdentityHashMap map2 = (WeakIdentityHashMap)val;
        map2.put(o, new Integer(o.hashCode()));
    }
    
    after(Hashtable t, Object o) returning(): scope() && remove(t, o) {
        WeakIdentityHashMap map2 = (WeakIdentityHashMap)map.get(t); /* object -> hashcode */
        if(map2!=null) {
            map2.remove(o);
        }
    }
    
    before(Hashtable t, Object o): scope() && use(t, o) {
        if(map.get(t)!=null) {
            WeakIdentityHashMap map2 = (WeakIdentityHashMap)map.get(t); /* object -> hashcode */
            Object code = map2.get(o);
            if(code!=null) {
                if(!( ((Integer)code).intValue()==o.hashCode()) ) {
                    System.out.println("HashCode of "+o+" changed while being in HashSet");
                }
            }
        }
    }
}
