import java.util.*;

public aspect RealHashtableWithLet
{
    pointcut use() :    call(* Map.get(..))
                     || call(* Map.remove(..))
                     || call(* Map.containsKey(..));

    /**
     * This version is better in a way that it checks if the
     * modification really changed the hash code of the
     * collection in question. Also it works for arbitrary
     * objects, not just collections.
     * BUT it uses let().
     */
    tracematch(Hashtable t, Object o, int hashCode)
    {
        sym put    before: call(* Map.put(Object, Object))
                                && target(t) && args(o,*)
                                && let(hashCode, o.hashCode());

        sym remove after:  call(* Map.remove(Object))
                                && target(t) && args(o);

        sym use    before: use() && target(t) && args(o);

        put use+
        {
            if(hashCode != o.hashCode())
                System.out.println("HashCode changed for Object " + o +
                                   " while being in a Hashtable.");
        }
    }
}
